@@ -3,19 +3,24 @@ from random import choice, sample
 import string
 
 from django.conf import settings
-from django.contrib import auth, messages
+from django.contrib import messages
+from django.contrib.auth import authenticate, login
 from django.contrib.auth.decorators import login_required
 from django.contrib.auth.models import User
 from django.core.mail import send_mail
 from django.core.urlresolvers import reverse
 from django.http import HttpResponseRedirect
-from django.shortcuts import get_object_or_404
+from django.shortcuts import get_object_or_404, render
 from django.template.defaultfilters import slugify
 
-from nano.tools import pop_error, render_page, get_profile_model, asciify
-from nano.user.forms import *
+from nano.tools import pop_error, get_profile_model, asciify
+from nano.user.forms import SignupForm, PasswordChangeForm, PasswordResetForm
 from nano.user import new_user_created
 
+
+import logging
+_LOG = logging.getLogger(__name__)
+
 Profile = get_profile_model(raise_on_error=False)
 
 class NanoUserError(Exception):
@@ -40,7 +45,7 @@ def random_password():
 
 def make_user(username, password, email=None, request=None):
     try:
-        user = User.objects.get(username=username)
+        User.objects.get(username=username)
     except User.DoesNotExist:
         # make user
         user = User(username=username[:30])
@@ -51,9 +56,13 @@ def make_user(username, password, email=None, request=None):
         if email:
             user.email = email
         user.save()
+        
+        # Create profile
         if Profile:
             profile = Profile(user=user, display_name=username)
             profile.save()
+
+        # Don't signal creation of test users
         test_users = getattr(settings, 'NANO_USER_TEST_USERS', ())
         for test_user in test_users:
             if user.username.startswith(test_user):
@@ -61,7 +70,9 @@ def make_user(username, password, email=None, request=None):
         else:
             new_user_created.send(sender=User, user=user) 
         if request is not None:
-            messages.info(request, u"You're now registered, as '%s'" % username)
+            infomsg = u"You're now registered, as '%s'" % username
+            messages.info(request, infomsg)
+            _LOG.debug('Created user: %s/%s' % (user, user.check_password(password)))
         return user
     else:
         raise NanoUserExistsError, "The username '%s' is already in use by somebody else" % username
@@ -81,27 +92,38 @@ def signup(request, template_name='signup.html', *args, **kwargs):
             password = form.cleaned_data['password2']
             email = form.cleaned_data['email'].strip() or ''
 
+            errormsg = u'Username "%s" is taken'
+
             # check that username not taken
             userslug = slugify(username)
             if Profile.objects.filter(slug=userslug).count():
                 # error!
                 safe_username = slugify('%s-%s' % (username, str(datetime.now())))
-                request.session['error'] = u"Username '%s' already taken, changed it to '%s'." % (username, safe_username)
+                changed_warningmsg = errormsg + ", changed it to '%s'."
+                messages.warning(request, changed_warningmsg % (username, safe_username))
                 username = safe_username
 
             # make user
-            user = make_user(username, password, email=email, request=request)
-            user = auth.authenticate(username=username, password=password)
-            auth.login(request, user)
-            request.session['error'] = None
-            next = getattr(settings, 'NANO_USER_SIGNUP_NEXT', reverse('nano_user_signup_done'))
             try:
+                user = make_user(username, password, email=email, request=request)
+            except NanoUserExistsError:
                 next_profile = user.get_profile().get_absolute_url()
                 return HttpResponseRedirect(next_profile)
-            except Profile.DoesNotExist:
-                pass
-            return HttpResponseRedirect(next)
-    return render_page(request, template_name, data)
+            else:
+                # fake authentication, avoid a db-lookup/thread-trouble/
+                # race conditions
+                user.backend = 'django.contrib.auth.backends.ModelBackend'
+                _LOG.debug('Attempting login of: %s' % user)
+                login(request, user)
+                nexthop = getattr(settings, 'NANO_USER_SIGNUP_NEXT', reverse('nano_user_signup_done'))
+                try:
+                    nexthop_profile = user.get_profile().get_absolute_url()
+                    return HttpResponseRedirect(nexthop_profile)
+                except Profile.DoesNotExist:
+                    pass
+                return HttpResponseRedirect(nexthop)
+            _LOG.debug('Should never end up here')
+    return render(request, template_name, data)
 
 @login_required
 def password_change(request, *args, **kwargs):
@@ -120,7 +142,7 @@ def password_change(request, *args, **kwargs):
         form = PasswordChangeForm()
     data = { 'form': form,
             'error': error,}
-    return render_page(request, template_name, data)
+    return render(request, template_name, data)
 
 def password_reset(request, project_name='Nano', *args, **kwargs):
     error = pop_error(request)
@@ -159,5 +181,5 @@ something you'll be able to remember.
     data = {'form': form,
             'help_message': help_message,
             'error':error}
-    return render_page(request, template, data)
+    return render(request, template, data)
 
